unit uEntities;

interface

uses
  SysUtils, 
  Generics.Collections, 
  Aurelius.Mapping.Attributes, 
  Aurelius.Types.Blob, 
  Aurelius.Types.DynamicProperties, 
  Aurelius.Types.Nullable, 
  Aurelius.Types.Proxy;

type
  TEnergiezaehler = class;
  TZaehlerstand = class;
  
  [Entity]
  [Table('Energiezaehler')]
  [Id('FIdEnergiezaehler', TIdGenerator.IdentityOrSequence)]
  TEnergiezaehler = class
  private
    [Column('Id_Energiezaehler', [TColumnProp.Required, TColumnProp.NoInsert, TColumnProp.NoUpdate])]
    FIdEnergiezaehler: Int64;
    
    [Column('Typ', [], 20)]
    FTyp: Nullable<string>;
    
    [Column('Bezeichnung', [], 50)]
    FBezeichnung: Nullable<string>;
    
    [Column('Zaehlernummer', [], 50)]
    FZaehlernummer: Nullable<string>;
    
    [Column('VerteilerMAC', [], 20)]
    FVerteilermac: Nullable<string>;
    
    [Column('CounterNr', [])]
    FCounternr: Nullable<Integer>;
    
    [Column('ImpulseJeEinh', [])]
    FImpulsejeeinh: Nullable<Integer>;
    
    [ManyValuedAssociation([TAssociationProp.Lazy], [TCascadeType.SaveUpdate, TCascadeType.Merge, TCascadeType.Remove], 'FEnergiezaehler')]
    FZaehlerstandList: Proxy<TList<TZaehlerstand>>;
    function GetZaehlerstandList: TList<TZaehlerstand>;
  public
    constructor Create;
    destructor Destroy; override;
    property IdEnergiezaehler: Int64 read FIdEnergiezaehler write FIdEnergiezaehler;
    property Typ: Nullable<string> read FTyp write FTyp;
    property Bezeichnung: Nullable<string> read FBezeichnung write FBezeichnung;
    property Zaehlernummer: Nullable<string> read FZaehlernummer write FZaehlernummer;
    property Verteilermac: Nullable<string> read FVerteilermac write FVerteilermac;
    property Counternr: Nullable<Integer> read FCounternr write FCounternr;
    property Impulsejeeinh: Nullable<Integer> read FImpulsejeeinh write FImpulsejeeinh;
    property ZaehlerstandList: TList<TZaehlerstand> read GetZaehlerstandList;
  end;
  
  [Entity]
  [Table('Zaehlerstand')]
  [Id('FIdZaehlerstand', TIdGenerator.IdentityOrSequence)]
  TZaehlerstand = class
  private
    [Column('Id_Zaehlerstand', [TColumnProp.Required, TColumnProp.NoInsert, TColumnProp.NoUpdate])]
    FIdZaehlerstand: Int64;
    
    [Column('Zeitpunkt', [TColumnProp.Required])]
    FZeitpunkt: TDateTime;
    
    [Column('Stand', [TColumnProp.Required], 18, 3)]
    FStand: Double;
    
    [Column('Counter', [TColumnProp.Required])]
    FCounter: Int64;
    
    [Column('ImpulseJeEinh', [TColumnProp.Required])]
    FImpulsejeeinh: Integer;
    
    [Association([TAssociationProp.Lazy], CascadeTypeAll - [TCascadeType.Remove])]
    [JoinColumn('Ref_Energiezaehler', [], 'Id_Energiezaehler')]
    FEnergiezaehler: Proxy<TEnergiezaehler>;
    function GetEnergiezaehler: TEnergiezaehler;
    procedure SetEnergiezaehler(const Value: TEnergiezaehler);
  public
    property IdZaehlerstand: Int64 read FIdZaehlerstand write FIdZaehlerstand;
    property Zeitpunkt: TDateTime read FZeitpunkt write FZeitpunkt;
    property Stand: Double read FStand write FStand;
    property Counter: Int64 read FCounter write FCounter;
    property Impulsejeeinh: Integer read FImpulsejeeinh write FImpulsejeeinh;
    property Energiezaehler: TEnergiezaehler read GetEnergiezaehler write SetEnergiezaehler;
  end;
  
implementation

{ TEnergiezaehler }

constructor TEnergiezaehler.Create;
begin
  inherited;
  FZaehlerstandList.SetInitialValue(TList<TZaehlerstand>.Create);
end;

destructor TEnergiezaehler.Destroy;
begin
  FZaehlerstandList.DestroyValue;
  inherited;
end;

function TEnergiezaehler.GetZaehlerstandList: TList<TZaehlerstand>;
begin
  result := FZaehlerstandList.Value;
end;

{ TZaehlerstand }

function TZaehlerstand.GetEnergiezaehler: TEnergiezaehler;
begin
  result := FEnergiezaehler.Value;
end;

procedure TZaehlerstand.SetEnergiezaehler(const Value: TEnergiezaehler);
begin
  FEnergiezaehler.Value := Value;
end;

initialization
  RegisterEntity(TEnergiezaehler);
  RegisterEntity(TZaehlerstand);

end.
